/**
 *
 * @author Ester Dalvit
 * dalvit@science.unitn.it
 */

import java.awt.Color;
import model.*;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import javax.swing.*;
import javax.media.opengl.GL;
import javax.media.opengl.glu.GLU;
import javax.media.opengl.glu.GLUquadric;
import java.awt.event.MouseWheelEvent;
import java.util.Vector;

class BraidModel extends Model {

    private static final GLU aglu = new GLU();
    GLUquadric quadric = aglu.gluNewQuadric();

    private Vector<Strand> strand;
    private Vector<Integer> word, leftCrossing, straightStrandIndex = new Vector<Integer>();
    private float movementTime, movementStep = 0.01f,
            stringRadius = 0.03f, cylinderRadius = 0.15f,
            braidHeight;
    private int fixingEnds, braidWidth;

    private float[][] aid3d, apxover, apxunder, anxover, anxunder,
            currentFloatColor;
    
    private int[] freeReducingIndex = {-1,-1}, handleIndex = {-1,-1},
            currentPermutation;

    private enum movement { STATIC, STRAIGHT_INCREASING, STRAIGHT_DECREASING, 
        HANDLE, INVERSE, WAIT};
    private movement currentMovement = movement.STATIC;

    private boolean freeze;
    private Color[] currentColor;

    private JButton resetButton, cancelButton,
            s1p,s1m,s2p,s2m,s3p,s3m,s4p,s4m,s5p,s5m,
            handleButton;

    // can pass these values from html with the same name
    static int MAX_ZOOM = 90, MIN_ZOOM = 5, INIT_ZOOM = 30,
            POINTS_IN_INTERVAL=5;
    static float[] endsColor = {0.65f, 0.65f, 0.45f }; 
    float light_ambient[] = { 0.4f, 0.4f, 0.4f }, 
        light_position[] = { 10.0f, 8.0f, 6.0f, 0f },
        light_diffuse[] = { 0.8f, 0.8f, 0.8f };

    BraidModel() {
        super();
        cameraX = 0;
        cameraY = 0;
        cameraZ = 3;
        lookAtX = 0;
        lookAtY = 0;
        lookAtZ = 0;
    }

    private void setWord() {
        word.add(1);
        word.add(4);
        word.add(1);
        word.add(3);
        word.add(2);
        word.add(3);
        word.add(2);
        word.add(-4);
        word.add(3);
        word.add(-4);
        word.add(-5);
    }

    void setPlay(boolean play) {
        if (play) {
            handleButton.setIcon((Icon)createImageIcon("images/play_t.png", ""));
            handleButton.setToolTipText("Start");
        }
        else {
            handleButton.setIcon((Icon)createImageIcon("images/freeze_t.png", ""));
            handleButton.setToolTipText("Stop");
        }
    }

    void setParameter(int index, int value) {
        int s;

        switch (index) {
            case 11:
                zoom.setZoom(value);
                break;

            case 688:
                word.clear();
                word.add(0);
                braidWidth = 1;
                braidHeight = 1f;

                for (s=0; s<strand.size(); s++)
                    strand.get(s).clear();
                strand.clear();

                initStrand();
                straightStrandIndex.clear();
                freeReducingIndex[0] = freeReducingIndex[1] = -1;
                handleIndex[1] = handleIndex[0] = -1;
                currentMovement = movement.STATIC;
                break;
                
            case 699:
                word.clear();
                setWord();
                initStrand();
                straightStrandIndex.clear();
                freeReducingIndex[0] = freeReducingIndex[1] = -1;
                handleIndex[1] = handleIndex[0] = -1;
                currentMovement = movement.STATIC;
                break;
//            case 700:
//                word.add(0);
//                calculateMinDim();
//                break;
            case 710:
                if (word.size()<=2 && word.get(0)==0)
                    word.clear();
                word.add(1);
                if (strand.size()<2) 
                    initStrand();
                else
                    calculateMinDim();
                break;
            case 711:
                if (word.size()<=2 && word.get(0)==0)
                    word.clear();
                word.add(-1);
                if (strand.size()<2)
                    initStrand();
                else
                    calculateMinDim();
                break;
            case 720:
                if (word.size()<=2 && word.get(0)==0)
                    word.clear();
                word.add(2);
                if (strand.size()<3)
                    initStrand();
                else
                    calculateMinDim();
                break;
            case 721:
                if (word.size()<=2 && word.get(0)==0)
                    word.clear();
                word.add(-2);
                if (strand.size()<3)
                    initStrand();
                else
                    calculateMinDim();
                break;
            case 730:
                if (word.size()<=2 && word.get(0)==0)
                    word.clear();
                word.add(3);
                if (strand.size()<4)
                    initStrand();
                else
                    calculateMinDim();
                break;
            case 731:
                if (word.size()<=2 && word.get(0)==0)
                    word.clear();
                word.add(-3);
                if (strand.size()<4)
                    initStrand();
                else
                    calculateMinDim();
                break;
            case 740:
                if (word.size()<=2 && word.get(0)==0)
                    word.clear();
                word.add(4);
                if (strand.size()<5)
                    initStrand();
                else
                    calculateMinDim();
                break;
            case 741:
                if (word.size()<=2 && word.get(0)==0)
                    word.clear();
                word.add(-4);
                if (strand.size()<5)
                    initStrand();
                else
                    calculateMinDim();
                break;
            case 750:
                if (word.size()<=2 && word.get(0)==0)
                    word.clear();
                word.add(5);
                if (strand.size()<6)
                    initStrand();
                else
                    calculateMinDim();
                break;
            case 751:
                if (word.size()<=2 && word.get(0)==0)
                    word.clear();
                word.add(-5);
                if (strand.size()<6)
                    initStrand();
                else
                    calculateMinDim();
                break;

            case 800:
                findStraightStrand();
                if (!straightStrandIndex.isEmpty()) {
                    movementTime = 0f;
                    currentMovement = movement.STRAIGHT_DECREASING;
                }
                break;
        }
    }
    
    @Override
    public void initLights(GL gl) {
        gl.glLightModeli(GL.GL_LIGHT_MODEL_TWO_SIDE, 1);
        gl.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, light_ambient, 0);
        gl.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, light_diffuse, 0);
        gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, light_position, 0);

        gl.glShadeModel(GL.GL_SMOOTH);
//        gl.glEnable(GL.GL_NORMALIZE); 
        gl.glEnable(GL.GL_LIGHTING);
        gl.glEnable(GL.GL_LIGHT0);
        gl.glEnable(GL.GL_DEPTH_TEST);
    }

    private void initStrand() {
        calculateMinDim();
        strand = new Vector<Strand>();
        for (int r=0; r<braidWidth; r++) {
            Strand s = new Strand();
            strand.addElement(s);
        }
        currentColor = new Color[braidWidth];

        for(int s=0; s<braidWidth; s++) {
            currentColor[s] = Color.getHSBColor(1f-(float)(s+1)/(float)braidWidth,0.9f,0.8f);
            strand.get(s).setFloatColor(currentColor[s]);
            currentFloatColor[s] = currentColor[s].getRGBColorComponents(currentFloatColor[s]);
        }
    }

    @Override
    public void init(GL gl) {         
        zoom.setMaxZoom(MAX_ZOOM);
        zoom.setMinZoom(MIN_ZOOM);
        zoom.setInitZoom(INIT_ZOOM);
        zoom.setZoom(INIT_ZOOM);
        
//        cylinderRadius = (float)INIT_CYL_RADIUS*0.003f;
        gl.glLineWidth(3f);
        gl.glPointSize(3f);
        gl.glEnable(GL.GL_COLOR_MATERIAL);
        gl.glColorMaterial(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT_AND_DIFFUSE);

        word = new Vector<Integer>();
        leftCrossing = new Vector<Integer>();
        setWord();
        initStrand();

        freeze = true;

        int i;
        float height;
        float[]vertex;
        
        aid3d = new float[POINTS_IN_INTERVAL][3];
        apxover = new float[POINTS_IN_INTERVAL][3];
        apxunder = new float[POINTS_IN_INTERVAL][3];
        anxover = new float[POINTS_IN_INTERVAL][3];
        anxunder = new float[POINTS_IN_INTERVAL][3]; 
        
        for (i=0; i<POINTS_IN_INTERVAL; i++) {
            height = (float)(i+1)/(float)POINTS_IN_INTERVAL;
            
            vertex = new float[3];
            vertex[0] = 0f;
            vertex[1] = -height;
            vertex[2] = 0f;
            aid3d[i] = vertex;
            
            vertex = calculateSplineOnCylinder(height,0f,(float)Math.PI, true);
            
            anxunder[i] = new float[3];
            anxunder[i][0] = vertex[0];
            anxunder[i][1] = vertex[1];
            anxunder[i][2] = vertex[2];
            
            anxover[i] = new float[3];
            anxover[i][0] = -vertex[0];
            anxover[i][1] = vertex[1];
            anxover[i][2] = -vertex[2];
            
            apxunder[i] = new float[3];
            apxunder[i][0] = -vertex[0];
            apxunder[i][1] = vertex[1];
            apxunder[i][2] = vertex[2];
            
            apxover[i] = new float[3];
            apxover[i][0] = vertex[0];
            apxover[i][1] = vertex[1];
            apxover[i][2] = -vertex[2];
        }

        fixingEnds = gl.glGenLists(1);
        gl.glNewList(fixingEnds, GL.GL_COMPILE);
        gl.glColor3fv(endsColor,0);
        gl.glBegin(GL.GL_POLYGON);
        gl.glVertex3f(0f,0f,-0.2f);
        gl.glVertex3f(0f,0f,0.2f);
        gl.glVertex3f(2f*cylinderRadius,0f,0.2f);
        gl.glVertex3f(2f*cylinderRadius,0f,-0.2f);
        gl.glEnd();
        gl.glBegin(GL.GL_POLYGON);
        gl.glVertex3f(0f,2f*stringRadius,-0.2f);
        gl.glVertex3f(0f,2f*stringRadius,0.2f);
        gl.glVertex3f(2f*cylinderRadius,2f*stringRadius,0.2f);
        gl.glVertex3f(2f*cylinderRadius,2f*stringRadius,-0.2f);
        gl.glEnd();
        gl.glBegin(GL.GL_POLYGON);
        gl.glVertex3f(0f,0f,-0.2f);
        gl.glVertex3f(0f,0f,0.2f);
        gl.glVertex3f(0f,2f*stringRadius,0.2f);
        gl.glVertex3f(0f,2f*stringRadius,-0.2f);
        gl.glEnd();
        gl.glBegin(GL.GL_POLYGON);
        gl.glVertex3f(0f,0f,0.2f);
        gl.glVertex3f(2f*cylinderRadius,0f,0.2f);
        gl.glVertex3f(2f*cylinderRadius,2f*stringRadius,0.2f);
        gl.glVertex3f(0f,2f*stringRadius,0.2f);
        gl.glEnd();
        gl.glBegin(GL.GL_POLYGON);
        gl.glVertex3f(2f*cylinderRadius,0f,0.2f);
        gl.glVertex3f(2f*cylinderRadius,0f,-0.2f);
        gl.glVertex3f(2f*cylinderRadius,2f*stringRadius,-0.2f);
        gl.glVertex3f(2f*cylinderRadius,2f*stringRadius,0.2f);
        gl.glEnd();
        gl.glBegin(GL.GL_POLYGON);
        gl.glVertex3f(2f*cylinderRadius,0f,-0.2f);
        gl.glVertex3f(0f,0f,-0.2f);
        gl.glVertex3f(0f,2f*stringRadius,-0.2f);
        gl.glVertex3f(2f*cylinderRadius,2f*stringRadius,-0.2f);
        gl.glEnd();
        gl.glEndList();
        
        calculateStaticBraid();
    }

    @Override
    public void redraw (GL gl) {
        // draw the boxes to "fix" braid ends
        gl.glPushMatrix();
        gl.glTranslatef(-((float)braidWidth)*cylinderRadius,1.5f,0f);
        gl.glPushMatrix();
        gl.glScalef((float)braidWidth,1f,1f);
        gl.glCallList(fixingEnds);
        gl.glTranslatef(0f,-3f-2f*stringRadius,0f);
        gl.glCallList(fixingEnds);
        gl.glPopMatrix();
        gl.glPopMatrix();


        // calculate the new braid
        gl.glScalef(1f,3f/braidHeight,1f);
        gl.glTranslatef(-((float)braidWidth)*cylinderRadius,braidHeight/2f,0f);

        if (!freeze) {
            switch (currentMovement) {
                case WAIT:
                      ActionListener taskPerformer = new ActionListener() {
                          public void actionPerformed(ActionEvent evt) {
                              handleButton.doClick();
                          }
                      };
                      Timer t = new Timer(1, taskPerformer);
                      t.setRepeats(false);
                      t.start();
                    break;

                case STRAIGHT_DECREASING: // decreasing the lenght of straight strands from 1 to 0
                    if (word.size()==2 && word.get(0) == -word.get(1)) {
                        currentMovement = movement.STATIC;
                        braidHeight = word.size();
                        straightStrandIndex.clear();
                    }
                    else {
                        movementTime += movementStep;

                        if (movementTime >= 1f) {
                            braidHeight -= movementStep;
                            movementTime = 0f;
                            deleteStraightStrand();
                            braidHeight = word.size();
                            currentMovement = movement.WAIT;
                        }
                    }
                    break;

                case STRAIGHT_INCREASING: // increasing the lenght of straight strands from 0 to 1
                    if (movementTime<=movementStep/2f) {
                        insertStraightStrand();
                    }
                    if (movementTime < 1f)
                        movementTime += movementStep;
                    else {
                        movementTime = 0f;
                        currentMovement = movement.HANDLE;
                        findcrossing();
                    }
                    break;

                case HANDLE:
                    movementTime += movementStep;

                    if (movementTime >= 1f) {
                        reduceHandle();
                        findStraightStrand();
                        currentMovement = movement.STRAIGHT_DECREASING;
                        movementTime = 0f;
                    }

                    break;

                case INVERSE: // simplifying two opposite crossings (a...A) into two straight lines
                    movementTime += movementStep;

                    if (movementTime >= 1f) {
                        reduceFree(freeReducingIndex);
                        straightStrandIndex.add(freeReducingIndex[0]);
                        straightStrandIndex.add(freeReducingIndex[1]);
                        currentMovement = movement.STRAIGHT_DECREASING;
                        freeReducingIndex[1] = -1;
                        movementTime = 0f;
                    }
                    break;
            }
        }

        recalculateBraid(gl);

        for (int s=0; s<strand.size(); s++)
            drawStrand(gl, s);
    }
    
      private void drawStrand(GL gl, int str) {
        int i;
        gl.glColor3fv(strand.get(str).getFloatColor(),0);

        for (i=1; i<strand.get(str).getLength()-1; i++) {
            gl.glPushMatrix();
            gl.glTranslatef(strand.get(str).getPoint(i)[0], strand.get(str).getPoint(i)[1], strand.get(str).getPoint(i)[2]);
            glu.gluSphere(quadric,stringRadius,10,10);
            gl.glPopMatrix();
        }

        for (i=1; i<strand.get(str).getLength(); i++)
            makeCylinder(gl, strand.get(str).getPoint(i-1), strand.get(str).getPoint(i), stringRadius);

    }

    private void calculateStaticBraid() { 
        int s,w,p, letter;
        float[]lastPoint;
        Strand currentStrand;
        int[] currentP = new int[braidWidth]; 
        
        for(s=0; s<braidWidth; s++) {
            strand.get(s).clear();
            float[]tmp = new float[3];
            tmp[0] = cylinderRadius*((float)s*2f+1f);
            strand.get(s).addPoint(0, tmp);
        }

        for (s=0; s<braidWidth; s++) {
            currentP[s] = s;
        }
        for(w=0; w<word.size(); w++) {
            letter = word.get(w);
            for(s=0; s<braidWidth; s++) {
                currentStrand = strand.get(currentP[s]);
                lastPoint = currentStrand.getPoints().lastElement();
                if (Math.abs(letter) == s) { 
                    if(letter>0) 
                        for(p=0; p<POINTS_IN_INTERVAL; p++) 
                            currentStrand.addPoint(add(lastPoint,apxover[p]));
                    else
                        for(p=0; p<POINTS_IN_INTERVAL; p++) 
                            currentStrand.addPoint(add(lastPoint,anxunder[p]));
                }
                else if (Math.abs(letter)-1 == s) {
                    if(letter>0)
                        for(p=0; p<POINTS_IN_INTERVAL; p++)
                            currentStrand.addPoint(add(lastPoint,apxunder[p]));
                    else
                        for(p=0; p<POINTS_IN_INTERVAL; p++)
                            currentStrand.addPoint(add(lastPoint,anxover[p]));
                }
                else
                    for(p=0; p<POINTS_IN_INTERVAL; p++) {
                        currentStrand.addPoint(add(lastPoint,aid3d[p]));
                    }
            }
            calculateNewPermutation(currentP, Math.abs(word.get(w)));
        }
    }
    
  
    private void recalculateBraid(GL gl) {
        int i,j,s,k,t;
        float currentLevel = 1f;
        float[]tmp, vertex2;
        float gap;
        
        for(s=0; s<braidWidth; s++) {
            strand.get(s).clear();
            tmp = new float[3];
            tmp[0] = cylinderRadius*((float)s*2f+1f);
            strand.get(s).addPoint(0, tmp);
        }

        for (j=0; j<braidWidth; j++) {
            currentPermutation[j] = j;
        }

        for (i=0; i<word.size(); i++) {
            gl.glPushMatrix();

            if (currentMovement == movement.STRAIGHT_DECREASING &&
                    straightStrandIndex.contains(i)) {
                for (j=0; j<braidWidth; j++) {
                    strand.get(currentPermutation[j]).addPoint(
                            (j*2f+1f)*cylinderRadius, -currentLevel+movementTime,0f);
                }

                if (!freeze) {
                    braidHeight -= movementStep;
                    currentLevel += movementStep;
                }     
            }
            
            else if (currentMovement == movement.STRAIGHT_INCREASING && 
                    straightStrandIndex.contains(i)) {
                for (j=0; j<braidWidth; j++) {
                    strand.get(currentPermutation[j]).addPoint(
                            (j*2f+1f)*cylinderRadius, -currentLevel+(1f-movementTime),0f);
                }
                if (!freeze) {
                    if (movementTime < 1f)
                        braidHeight += movementStep;
                    else
                        braidHeight = word.size();
                    currentLevel += -1f-movementStep;
                }
                else
                    currentLevel -= 1f;
            }

            else if (currentMovement == movement.INVERSE) {
                if (i == freeReducingIndex[0]) {

                    // draw straight strands on the left
                    for (j=1; j<Math.abs(word.get(i)); j++) {
                        strand.get(currentPermutation[j-1]).addPoint(
                                (j*2f-1f)*cylinderRadius, -currentLevel, 0f);
                    }

                    // draw the upper crossing which is changing to identity                    
                    if (word.get(i)<0) {
                        for (k=POINTS_IN_INTERVAL; k>=0; k--) {
                            gap = (float)(k)/(float)POINTS_IN_INTERVAL;
                            vertex2 = calculateSplineOnCylinder(gap,(float)Math.PI*(1f-movementTime),0f, false);

                            tmp = new float[3];
                            tmp[0] = (j*2f)*cylinderRadius + vertex2[0];
                            tmp[1] = -currentLevel + vertex2[1];
                            tmp[2] = vertex2[2];
                            strand.get(currentPermutation[j]).addPoint(tmp);

                            tmp = new float[3];
                            tmp[0] = (j*2f)*cylinderRadius - vertex2[0];
                            tmp[1] = -currentLevel + vertex2[1];
                            tmp[2] = -vertex2[2];
                            strand.get(currentPermutation[j-1]).addPoint(tmp);
                        }
                    }
                    else {
                        for (k=POINTS_IN_INTERVAL; k>=0; k--) {
                            gap = (float)(k)/(float)POINTS_IN_INTERVAL;
                            vertex2 = calculateSplineOnCylinder(gap,(float)Math.PI*(-1f+movementTime),0f, false); 

                            tmp = new float[3];
                            tmp[0] = (j*2f)*cylinderRadius + vertex2[0];
                            tmp[1] = -currentLevel + vertex2[1];
                            tmp[2] = vertex2[2];
                            strand.get(currentPermutation[j]).addPoint(tmp);

                            tmp = new float[3];
                            tmp[0] = (j*2f)*cylinderRadius - vertex2[0];
                            tmp[1] = -currentLevel + vertex2[1];
                            tmp[2] = -vertex2[2];
                            strand.get(currentPermutation[j-1]).addPoint(tmp);
                        }
                    }

                    // draw straight strands on the right
                    for (j=Math.abs(word.get(i))+1; j<braidWidth; j++) {
                        strand.get(currentPermutation[j]).addPoint( 
                                (j*2f+1f)*cylinderRadius, -currentLevel, 0f);
                    }
                }
                else if (i == freeReducingIndex[1]) {
                    // draw straight strands on the left
                    for (j=1; j<Math.abs(word.get(i)); j++) {
                        strand.get(currentPermutation[j-1]).addPoint(
                                (j*2f-1f)*cylinderRadius, -currentLevel, 0f);
                    }
                    // draw the lower crossing which is changing to identity
                    if (word.get(i)<0) { 
                        for (k=POINTS_IN_INTERVAL; k>=0; k--) {
                            gap = (float)(k)/(float)POINTS_IN_INTERVAL;
                            vertex2 = calculateSplineOnCylinder(gap,0f,(float)Math.PI*(-1f+movementTime), false); 

                            tmp = new float[3];
                            tmp[0] = (j*2f)*cylinderRadius + vertex2[0];
                            tmp[1] = -currentLevel + vertex2[1];
                            tmp[2] = vertex2[2];
                            strand.get(currentPermutation[j-1]).addPoint(tmp);

                            tmp = new float[3];
                            tmp[0] = (j*2f)*cylinderRadius - vertex2[0];
                            tmp[1] = -currentLevel + vertex2[1];
                            tmp[2] = -vertex2[2];
                            strand.get(currentPermutation[j]).addPoint(tmp);
                        }
                    }
                    else {
                        for (k=POINTS_IN_INTERVAL; k>=0; k--) {
                            gap = (float)(k)/(float)POINTS_IN_INTERVAL;
                            vertex2 = calculateSplineOnCylinder(gap,0f,(float)Math.PI*(1f-movementTime), false);

                            tmp = new float[3];
                            tmp[0] = (j*2f)*cylinderRadius + vertex2[0];
                            tmp[1] = -currentLevel + vertex2[1];
                            tmp[2] = vertex2[2];
                            strand.get(currentPermutation[j-1]).addPoint(tmp);

                            tmp = new float[3];
                            tmp[0] = (j*2f)*cylinderRadius - vertex2[0];
                            tmp[1] = -currentLevel + vertex2[1];
                            tmp[2] = -vertex2[2];
                            strand.get(currentPermutation[j]).addPoint(tmp);
                        }
                    }

                    // draw straight strands on the right
                    for (j=Math.abs(word.get(i))+1; j<braidWidth; j++) {
                        strand.get(currentPermutation[j]).addPoint( 
                                (j*2f+1f)*cylinderRadius, -currentLevel, 0f);
                    }
                }

                else if (i > freeReducingIndex[0] && i < freeReducingIndex[1]) {
                    for (j=0; j<Math.abs(word.get(freeReducingIndex[0]))-1; j++) {
                        drawStaticBlock(gl, i, j, currentLevel);
                    }
                    for (j=Math.abs(word.get(freeReducingIndex[0]))+1; j<braidWidth; j++) { //draw static piece
                        drawStaticBlock(gl, i, j, currentLevel);
                    }
                }
                else {
                    drawStaticLevel(gl,i, currentLevel);
                }
            }
            
            else if (currentMovement == movement.HANDLE) {
                boolean posHandle = false;

                if (signum(word.get(handleIndex[0])) > 0)
                    posHandle = true;

                if (i<handleIndex[0]) // over the handle
                    drawStaticLevel(gl, i, currentLevel);
                else if (i<=handleIndex[1]) {
                    // draw braid on the left of the handle
                    for (j=0; j<Math.abs(word.get(handleIndex[0]))-2; j++) // left
                        drawStaticBlock(gl, i, j, currentLevel);
                    
                    // draw crossings 2 positions left of handle
                    t=Math.abs(word.get(handleIndex[0]))-2; 
                    if (Math.abs(word.get(i)) == t)
                        drawStaticBlock(gl, i, t, currentLevel);
                    
                    if (i==handleIndex[0]) { // braid rel
                        // strand 0
                        strand.get(currentPermutation[j]).addPoint(
                                (j*2f+1f)*cylinderRadius, -currentLevel, 0f);
                        
                        // strand 2
                                for (t=0; t<=POINTS_IN_INTERVAL; t++) { 
                                    gap = (float)(t)/(float)POINTS_IN_INTERVAL;
                                    vertex2 = calculateSplineOnCircle(gap,movementTime,true, true, posHandle);
                                    tmp = new float[3];
                                    tmp[0] = (j*2f+5f)*cylinderRadius + vertex2[0];
                                    tmp[1] = -currentLevel + vertex2[1];
                                    tmp[2] = vertex2[2];
                                    strand.get(currentPermutation[j+2]).addPoint(tmp);
                                }

                        // strand 1 (handle)
                        // straight piece of the handle, over
                        strand.get(currentPermutation[j+1]).addPoint(
                                (j*2f+3f)*cylinderRadius,
                                -currentLevel-movementTime*(straightStrandIndex.firstElement()-handleIndex[0])+1f, 0);

                        // piece of the handle on the cylinder
                                for (t=0; t<=POINTS_IN_INTERVAL; t++) { 
                                    gap = (float)(POINTS_IN_INTERVAL-t)/(float)POINTS_IN_INTERVAL;
                                    if (movementTime<0.5f)
                                        vertex2 = calculateSplineOnBigCylinder(1f-gap,movementTime*2f,false,false,posHandle);
                                    else
                                        vertex2 = calculateSplineOnBigCylinder(1f-gap,2f-movementTime*2f,true,false,posHandle);
                                    tmp = new float[3];
                                    tmp[0] = (j + 1.5f)*2f*cylinderRadius + vertex2[0];
                                    tmp[1] = -currentLevel-movementTime*(straightStrandIndex.firstElement()-handleIndex[0]) + vertex2[1];
                                    tmp[2] = vertex2[2];
                                    strand.get(currentPermutation[j+1]).addPoint(tmp);
                                }
                    }

                    else if (i<handleIndex[1]) {
                        if (leftCrossing.contains(i+1)) { // level before a crossing just left of the handle
                                for (t=0; t<=POINTS_IN_INTERVAL; t++) {
                                    gap = (float)(t)/(float)POINTS_IN_INTERVAL;
                                    vertex2 = calculateSplineOnCircle(gap,1f-movementTime, false, true, posHandle);
                                    tmp = new float[3];
                                    tmp[0] = (j+0.5f)*2f*cylinderRadius + vertex2[0];
                                    tmp[1] = -currentLevel + vertex2[1];
                                    tmp[2] = vertex2[2];
                                    strand.get(currentPermutation[j]).addPoint(tmp);
                                } 

                            strand.get(currentPermutation[j+1]).addPoint(
                                    ((j+2f+movementTime)*2f-1f)*cylinderRadius, -currentLevel, 0);
                            

                            if (movementTime>=0.5f) {
                                if (!(leftCrossing.lastElement()==i+1)) {

                                    for (t=0; t<=POINTS_IN_INTERVAL; t++) { 
                                        gap = (float)(t)/(float)POINTS_IN_INTERVAL;
                                        vertex2 = calculateSplineHandle(gap,movementTime*2f-1f,false, false, posHandle);
                                        tmp = new float[3];
                                        tmp[0] = ((2f*j+5f) - movementTime*4f)*cylinderRadius + vertex2[0];
                                        tmp[1] = -currentLevel-1f-movementTime + vertex2[1];
                                        if (leftCrossing.firstElement()!=i+1)
                                            tmp[1] -= 1f;
                                        tmp[2] = vertex2[2];
                                        strand.get(currentPermutation[j+2]).addPoint(tmp);
                                    }  
                                }
                            }
                        }
                        else if (leftCrossing.contains(i)) { // level of a crossing to be shifted to the right
                                if (word.get(i)>0) {
                                        for (t=POINTS_IN_INTERVAL; t>=0; t--) { 
                                        gap = (float)(t)/(float)POINTS_IN_INTERVAL;
                                        vertex2 = calculateSplineOnCylinder(gap,0f,(float)Math.PI, false);
                                        tmp = new float[3];
                                        tmp[0] = (j+1+movementTime)*2f*cylinderRadius + vertex2[0];
                                        tmp[1] = -currentLevel + vertex2[1];
                                        tmp[2] = vertex2[2];
                                        strand.get(currentPermutation[j]).addPoint(tmp);

                                        tmp = new float[3];
                                        tmp[0] = (j+1+movementTime)*2f*cylinderRadius - vertex2[0];
                                        tmp[1] = -currentLevel + vertex2[1];
                                        tmp[2] = -vertex2[2];
                                        strand.get(currentPermutation[j+1]).addPoint(tmp);
                                    }
                                }
                                else {
                                    for (t=POINTS_IN_INTERVAL; t>=0; t--) { 
                                        gap = (float)(t)/(float)POINTS_IN_INTERVAL;
                                        vertex2 = calculateSplineOnCylinder(gap,(float)Math.PI,0f, false);

                                        tmp = new float[3];
                                        tmp[0] = (j+1+movementTime)*2f*cylinderRadius + vertex2[0];
                                        tmp[1] = -currentLevel + vertex2[1];
                                        tmp[2] = vertex2[2];
                                        strand.get(currentPermutation[j+1]).addPoint(tmp);

                                        tmp = new float[3];
                                        tmp[0] = (j+1+movementTime)*2f*cylinderRadius - vertex2[0];
                                        tmp[1] = -currentLevel + vertex2[1];
                                        tmp[2] = -vertex2[2];
                                        strand.get(currentPermutation[j]).addPoint(tmp);
                                    }
                                }
                        }

                        else if (leftCrossing.contains(i-1)) { // level after a crossing just left of the handle
                            for (t=0; t<=POINTS_IN_INTERVAL; t++) { 
                                        gap = (float)(t)/(float)POINTS_IN_INTERVAL;
                                        vertex2 = calculateSplineOnCircle(gap, 1f-movementTime, true, true, posHandle);
                                        tmp = new float[3];
                                        tmp[0] = (j+0.5f+movementTime)*2f*cylinderRadius + vertex2[0];
                                        tmp[1] = -currentLevel + vertex2[1];
                                        tmp[2] = vertex2[2];
                                        strand.get(currentPermutation[j]).addPoint(tmp);
                                    }
                            strand.get(currentPermutation[j+1]).addPoint(
                                    ((j+2f+movementTime)*2f-1f)*cylinderRadius, -currentLevel, 0); 
                            

                            if (movementTime>=0.5f) {
                                if (!(leftCrossing.firstElement()==i-1)) {
                                    for (t=0; t<=POINTS_IN_INTERVAL; t++) {
                                        gap = (float)(POINTS_IN_INTERVAL-t)/(float)POINTS_IN_INTERVAL;
                                        vertex2 = calculateSplineHandle(gap,movementTime*2f-1f,false, true, posHandle);
                                        tmp = new float[3];
                                        tmp[0] = ((2f*j+5f) - movementTime*4f)*cylinderRadius + vertex2[0];
                                        tmp[1] = -(currentLevel)+1f+movementTime + vertex2[1];
                                        tmp[2] = vertex2[2];
                                        strand.get(currentPermutation[j+2]).addPoint(tmp);
                                    }
                                }
                            }
                        }

                    }

                    else { // i==handleIndex[1]
                        // strand 0
                        // needed if the handle ends in the end of the braid
                        strand.get(currentPermutation[j]).addPoint(
                                (j*2f+1f)*cylinderRadius, -currentLevel, 0f);

                                for (t=0; t<=POINTS_IN_INTERVAL; t++) {
                                    gap = (float)(t)/(float)POINTS_IN_INTERVAL;
                                    vertex2 = calculateSplineOnCircle(gap,movementTime,false, true, posHandle);
                                    tmp = new float[3];
                                    tmp[0] = (j+1.5f+movementTime)*2f*cylinderRadius + vertex2[0];
                                    tmp[1] = -currentLevel + vertex2[1];
                                    tmp[2] = vertex2[2];
                                    strand.get(currentPermutation[j+1]).addPoint(tmp);
                                }

                        // the arc rotating on the cylinder with double radius
                                for (t=0; t<=POINTS_IN_INTERVAL; t++) {
                                    gap = (float)(t)/(float)POINTS_IN_INTERVAL;
                                    if (movementTime<0.5)
                                        vertex2 = calculateSplineOnBigCylinder(1f-gap,2f*movementTime,false,true,posHandle);
                                    else
                                        vertex2 = calculateSplineOnBigCylinder(1f-gap,2f-movementTime*2f,true,true,posHandle);
                                    tmp = new float[3];
                                    tmp[0] = (j + 1.5f)*2f*cylinderRadius + vertex2[0];
                                    tmp[1] = -currentLevel+movementTime*(handleIndex[1]-straightStrandIndex.lastElement()) + vertex2[1];
                                    tmp[2] = vertex2[2];
                                    strand.get(currentPermutation[j+2]).addPoint(tmp);
                                } 

                        strand.get(currentPermutation[j+2]).addPoint(  
                                ((j+2f)*2f-1f)*cylinderRadius, -currentLevel, 0);
                    }

                    for (j=Math.abs(word.get(handleIndex[0]))+1; j<braidWidth; j++) { // right
                        drawStaticBlock(gl, i, j, currentLevel);
                    }

                }
                else // levels below the handle
                    drawStaticLevel(gl, i, currentLevel); 
            }
            
            else { // if currentMovement = movement.STATIC
                drawStaticLevel(gl,i,currentLevel);
            }

            
            currentLevel += 1f;
            if (currentMovement == movement.STRAIGHT_DECREASING &&
                    straightStrandIndex.contains(i)) {
                currentLevel -= movementTime;
                if (freeze)
                    currentLevel -=movementStep;
            }
            else if (currentMovement == movement.STRAIGHT_INCREASING &&
                    straightStrandIndex.contains(i))
                currentLevel += movementTime;
            
            gl.glPopMatrix();

            if (word.get(i) != 0)
                calculateNewPermutation(currentPermutation, Math.abs(word.get(i)));
        }
    }

    private void drawStaticLevel(GL gl, int i, float currentLevel) {
        int j;
        for (j=0; j<braidWidth; j++)
            drawStaticBlock(gl, i, j, currentLevel);
    }

    // i=level, j=strand
    private void drawStaticBlock(GL gl, int i, int j, float currentLevel) {
        float[]tmp, vertex;
        int t;
        float gap;
        if (word.get(i)==0 || 
                (j != Math.abs(word.get(i)) && j != Math.abs(word.get(i))-1)) {
            strand.get(currentPermutation[j]).addPoint( 
                    (j*2f+1f)*cylinderRadius, -currentLevel, 0f);
        }
        else if (j == Math.abs(word.get(i))) {
            gl.glPushMatrix();
            gl.glTranslatef(j*2f*cylinderRadius, -currentLevel, 0);
                if (word.get(i)>0) {
                    for (t=POINTS_IN_INTERVAL; t>=0; t--) {
                        gap = (float)(t)/(float)POINTS_IN_INTERVAL;
                        vertex = calculateSplineOnCylinder(gap,0f,(float)Math.PI, false);

                        tmp = new float[3];
                        tmp[0] = j*2f*cylinderRadius + vertex[0];
                        tmp[1] = -currentLevel + vertex[1];
                        tmp[2] = vertex[2];
                        strand.get(currentPermutation[j-1]).addPoint(tmp);

                        tmp = new float[3];
                        tmp[0] = j*2f*cylinderRadius - vertex[0];
                        tmp[1] = -currentLevel + vertex[1];
                        tmp[2] = -vertex[2];
                        strand.get(currentPermutation[j]).addPoint(tmp);
                    }
                }
                else {
                    for (t=POINTS_IN_INTERVAL; t>=0; t--) {
                        gap = (float)(t)/(float)POINTS_IN_INTERVAL;
                        vertex = calculateSplineOnCylinder(gap,(float)Math.PI,0f, false);

                        tmp = new float[3];
                        tmp[0] = j*2f*cylinderRadius + vertex[0];
                        tmp[1] = -currentLevel + vertex[1];
                        tmp[2] = vertex[2];
                        strand.get(currentPermutation[j]).addPoint(tmp);

                        tmp = new float[3];
                        tmp[0] = j*2f*cylinderRadius - vertex[0];
                        tmp[1] = -currentLevel + vertex[1];
                        tmp[2] = -vertex[2];
                        strand.get(currentPermutation[j-1]).addPoint(tmp);
                    }
                }
            gl.glPopMatrix();
        }
    }

    private void calculateNewPermutation(int[]currentP, int s) {
        int tmp = currentP[s];
        currentP[s] = currentP[s-1];
        currentP[s-1] = tmp;
    }

    private void findcrossing() {
        int i;
        leftCrossing.clear();
        for (i=handleIndex[0]+1; i<handleIndex[1]; i++) {
            if (Math.abs(word.get(i)) == Math.abs(word.get(handleIndex[0]))-1) {
                leftCrossing.addElement(i);
            }
        }
    }

    private void printWord() {
        int i;
        for (i=0; i<word.size(); i++)
            System.out.print(word.get(i) + " ");
        System.out.print("\n");
    }

    // cubic spline on the cylinder. startAngle, endAngle in [0,2pi]
    // for positive crossing: startAngle is 0, endAngle is pi
    // for positive crossing: startAngle is pi, endAngle is 0
    // to calculate geodesic put angle = height*(endAngle-startAngle)
    private float[] calculateSplineOnCylinder(float height, float startAngle, float endAngle, boolean forstrand) {
        float[]pointOnCylinder = new float[3];
        float angle = (float)(2.0*(startAngle-endAngle)*Math.pow(height, 3.0) +
                3.0*(endAngle-startAngle)*Math.pow(height, 2.0) + startAngle);
      
        pointOnCylinder[0] = forstrand ? cylinderRadius*((float)Math.cos(angle)-1f) : cylinderRadius*(float)Math.cos(angle);
        pointOnCylinder[1] = forstrand ? -height: height;
        pointOnCylinder[2] = -cylinderRadius*(float)Math.sin(angle);
        
        return pointOnCylinder;
    }

    // used in the move for the braid rel +x-
    // draws the spline on a cylinder with radius twice the cylinderRadius
    // p is the parameter to draw the curve [0,1]
    // time controls the movement of the curve on the big cylinder [0,1]
    private float[] calculateSplineOnBigCylinder(float p, float time, boolean reverseX, boolean reverseY, boolean reverseZ) {
        float[]pointToDraw = new float[3];
        float splinep = (float)(- 2.0*Math.pow(p, 3.0) + 3.0*Math.pow(p, 2.0));

        if (reverseX)
            pointToDraw[0] = -cylinderRadius*(1f-time)*(1f-(float)Math.cos(splinep*Math.PI));
        else
            pointToDraw[0] = cylinderRadius*(1f-time)*(1f-(float)Math.cos(splinep*Math.PI));
        if (reverseY)
            pointToDraw[1] = p;
        else
            pointToDraw[1] = -p+1f;
        if (reverseZ)
            pointToDraw[2] = -cylinderRadius * ( 2f*splinep*time + (1f-time) * (float)Math.sin(splinep*Math.PI));
        else
            pointToDraw[2] = cylinderRadius * ( 2f*splinep*time + (1f-time) * (float)Math.sin(splinep*Math.PI));

        return pointToDraw;
    }

    // used in the move for the braid rel +x-
    // draws the spline on a circle with increasing radius (from 0 to cylinderRadius)
    // p is the parameter to draw the curve [0,1]
    // time controls the movement of the curve on the plane [0,1]
    private float[] calculateSplineOnCircle(float p, float time, boolean reverseX, boolean reverseY, boolean reverseZ) {
        float[]pointToDraw = new float[3];
        float splinep = (float)(- 2.0*Math.pow(p, 3.0) + 3.0*Math.pow(p, 2.0));

        if (!reverseX)
            pointToDraw[0] = (1f-time)*cylinderRadius* (1-(float)Math.cos(splinep*Math.PI) );
        else
            pointToDraw[0] = -(1f-time)*cylinderRadius* (1-(float)Math.cos(splinep*Math.PI) ); 
        if (!reverseY)
            pointToDraw[1] = p;
        else
            pointToDraw[1] = -p+1f;
        if(reverseZ)
            pointToDraw[2] = (1f-time)*cylinderRadius* (float)Math.sin(splinep*Math.PI);
        else
            pointToDraw[2] = -(1f-time)*cylinderRadius* (float)Math.sin(splinep*Math.PI);

        return pointToDraw;
    }


    private float[] calculateSplineHandle(float p, float time, boolean reverseX, boolean reverseY, boolean reverseZ) {
        float[]pointToDraw = new float[3];
        float splinep = (float)(- 2.0*Math.pow(p, 3.0) + 3.0*Math.pow(p, 2.0));

        if (reverseX)
            pointToDraw[0] = time*cylinderRadius*(-1f+(float)Math.cos(splinep*Math.PI));
        else
            pointToDraw[0] = -time*cylinderRadius*(-1f+(float)Math.cos(splinep*Math.PI));
        if (reverseY)
            pointToDraw[1] = p;
        else
            pointToDraw[1] = -p+1f;
        if (reverseZ)
            pointToDraw[2] = -cylinderRadius * ( 2f*(1f-time) + (time) * (float)Math.sin(splinep*Math.PI));
        else
            pointToDraw[2] = cylinderRadius * ( 2f*(1f-time) + (time) * (float)Math.sin(splinep*Math.PI));

        return pointToDraw;
    }


    private void findHandle() {
        // linear in time
        // finds more than handles
        
        int[]sign = new int[braidWidth+1],
                lastoccurrence = new int[braidWidth+1]; 
        int i, currentLetter;
        boolean found = false;

        for (i=0; i<braidWidth+1; i++)
            lastoccurrence[i] = -1;

        for (i=0; i<word.size() && !found; i++) {
            currentLetter = word.get(i);
            if (Math.abs(currentLetter) > 0) {
                sign[Math.abs(currentLetter)-1] = 0;
//                lastoccurrence[Math.abs(currentLetter)-1] = -1;
            }
            if (sign[Math.abs(currentLetter)] == -signum(currentLetter)) {
                handleIndex[0] = lastoccurrence[Math.abs(currentLetter)];
                handleIndex[1] = i;
//                printWord();
                if (Math.abs(currentLetter)==1 ||
                        lastoccurrence[Math.abs(currentLetter)-1] < handleIndex[0]) {
//                    System.out.println("trivial handle: lo of " + (Math.abs(currentLetter)-1)
//                            + " is " + lastoccurrence[Math.abs(currentLetter)-1]
//                            + " handleIndex[0] is " + handleIndex[0]);
                    freeReducingIndex[0] = handleIndex[0];
                    freeReducingIndex[1] = handleIndex[1];
                    currentMovement = movement.INVERSE;
                    movementTime = 0f;
                }
                else {
//                    System.out.println("non trivial handle");
                    initializeHR();
                }
                found = true;
                
//                System.out.println("linear: " + handleIndex[0] + ", " + handleIndex[1]);
                break;
            }
            else {
                sign[Math.abs(currentLetter)] = signum(currentLetter);
                lastoccurrence[Math.abs(currentLetter)] = i;
            }
        }
        if (!found)
            handleIndex[1] = -1;
    }

    private void initializeHR() {
        int i;
        straightStrandIndex.clear();

        for (i=handleIndex[0]+1; i<handleIndex[1]; i++) {
            if (Math.abs(word.get(i)) == Math.abs(word.get(handleIndex[0]))-1) {
                straightStrandIndex.add(i); 
                straightStrandIndex.add(i+1); 
                movementTime = 0f;
                currentMovement = movement.STRAIGHT_INCREASING;
            }
        }
    }

    private void findStraightStrand() { // find 0s in the word
        int i;
        straightStrandIndex.clear();
        for (i=0; i<word.size(); i++)
            if (word.get(i) == 0)
                straightStrandIndex.add(i);
    }

    private void deleteStraightStrand() { 
        int i;
        
        for (i=straightStrandIndex.size()-1; i>=0; i--) {
//            printWord();
//            System.out.println("remove element # " + tmp[i]);
            word.removeElementAt(straightStrandIndex.get(i));
        }
        straightStrandIndex.clear();
    }

    private void insertStraightStrand() {
        int i;
        for (i=straightStrandIndex.size()-1; i>=0; i--) {
            word.add(straightStrandIndex.get(i),0);
        }
        for (i=0; i<straightStrandIndex.size(); i++) {
            straightStrandIndex.set(i, straightStrandIndex.get(i)+i);
            handleIndex[1]++;
        }
    }

    private void reduceHandle() {
        int i, 
                crossing = Math.abs(word.get(handleIndex[0])),
                sign = signum(word.get(handleIndex[0])),
                sign2;

//        System.out.println("reduce Handle");
//        printWord();
        for (i=handleIndex[0]+1; i<handleIndex[1]; i++) {
            sign2 = signum(word.get(i+1));
            if (word.get(i)==0 && word.get(i+2)==0) {
                word.set(i, (crossing-1)*(-sign));
                word.set(i+2, (crossing-1)*(sign));
//                System.out.println("crossing " + crossing);
                word.set(i+1, sign2*crossing);
                i+=2;
            }
        }
        word.set(handleIndex[0], 0);
        word.set(handleIndex[1], 0);
//        leftCrossing.clear();

//        printWord();
    }

    private void reduceFree(int[] indices) {
        word.set(indices[0],0);
        word.set(indices[1],0);
    }


    private int signum(int a) {
        if (a<0)
            return -1;
        if (a>0)
            return 1;
        return 0;
    }

    private void calculateMinDim() {
        int r=0;
        for (int i=0; i<word.size(); i++)
            if (Math.abs(word.get(i)) > r)
                r = Math.abs(word.get(i));
        if (braidWidth<r+1)
            braidWidth = r+1;
        braidHeight = (float)word.size();
        currentPermutation = new int[braidWidth];
        currentColor = new Color[braidWidth];
        currentFloatColor = new float[braidWidth][3];
        for (r=0; r<braidWidth; r++)
            currentPermutation[r] = r;
//        initStrandColor();
    }
    
    public void makeCylinder(GL gl, float[]p, float[]q, float r) {
        makeCylinder(gl,p[0],p[1],p[2],q[0],q[1],q[2],r);
    }

    public void makeCylinder(GL gl, float p0, float p1, float p2,
            float q0, float q1, float q2, float r) {
        // initial point P, final point Q
        gl.glPushMatrix();
        float l = (float)Math.sqrt(Math.pow(q0-p0,2.0)+Math.pow(q1-p1,2.0)+Math.pow(q2-p2,2.0));
        float m = (float)Math.sqrt(Math.pow(q0-p0,2.0)+Math.pow(q1-p1,2.0)+Math.pow(q2-p2-l,2.0));
        float theta = (float)(2f * 180f / Math.PI * Math.asin( m / (2.0 * l)));
        gl.glTranslatef(p0,p1,p2);
        gl.glRotatef(-theta, q1-p1, -q0+p0, 0.0f);
        glu.gluCylinder(quadric, r, r, l, 10, 10);
//        makeBaseCylinder(gl, r, l, 20);
        gl.glPopMatrix();
    }
    
    
    private float[] add(float[]a,float[]b) {
        float[]result = new float[3];
        result[0] = a[0] + b[0];
        result[1] = a[1] + b[1];
        result[2] = a[2] + b[2];
        return result;
    }
    
    protected static ImageIcon createImageIcon(String path, String description) {
        java.net.URL imgURL = BraidModel.class.getResource(path);

        if (imgURL != null) {
            return new ImageIcon(imgURL, description);
        } else {
            System.err.println("Couldn't find file: " + path);
            return null;
        }
    }

    public JPanel createButton(JButton b, int index, String tipText ) {
        final int ind = index;
        
        Dimension d = new Dimension(55,30);
        
        b.setVerticalTextPosition(AbstractButton.CENTER);
        b.setHorizontalTextPosition(AbstractButton.CENTER);
        b.setToolTipText(tipText);
        
        b.setMinimumSize(d);
        b.setMaximumSize(d);
        b.setPreferredSize(d);
        
        b.addActionListener(
            new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (currentMovement == movement.STATIC) {
                    if (ind==688 || ind==699) {
                        freeze = false;
                        setPlay(true);
                        setParameter(ind,0);
                    }
                    else if (ind==840) {
                        if (word.size()!=1) {
                            findHandle();
                            if (handleIndex[1] != -1) {
                                movementTime = 0f;
                                freeze = false;
                                setPlay(false);
                            }
                            else 
                                setPlay(true);
                        }
                        else {
                            freeze = false;
                            setPlay(true);
                        }
                    }
                    else {
                        setParameter(ind, 0);
                    }
                }
                else if (currentMovement == movement.WAIT) {
                    if (ind==688 || ind==699) {
                        freeze = false;
                        setPlay(true);
                        setParameter(ind,0);
                    }
                    else if (ind==840) {
                        if (word.size()!=1) { 
                            findHandle();
                            if (handleIndex[1] != -1) {
                                movementTime = 0f;
                                freeze = false;
                                setPlay(false);
                            }
                            else {
                                setPlay(true);
                                currentMovement = movement.STATIC;
                            }
                        }
                        else {
                            freeze = false;
                            setPlay(true);
                            currentMovement = movement.STATIC;
                        }
                    }
                }
                else if (ind==688 || ind==699) { //  is moving
                    setParameter(ind,0);
                    freeze = false;
                    setPlay(true);
                }
                else if (ind==840) { //  is moving
                    if (freeze) {
                        freeze = false;
                        setPlay(false);
                    }
                    else {
                        freeze = true;
                        setPlay(true);
                    }
                }
            }
        });
        
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.LINE_AXIS));
        panel.add(b);
        
        return panel;
    }   
 
    @Override
    public JPanel createIndex() {
        JPanel panel= new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.PAGE_AXIS));
        panel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

//        JPanel nameL = new JPanel();
//        nameL.add(new JLabel("Braid"), BoxLayout.X_AXIS);
//        panel.add(nameL);

        JPanel generatorsPanel = new JPanel();
        generatorsPanel.setLayout(new BoxLayout(generatorsPanel, BoxLayout.Y_AXIS));
//        s0 = new JButton("0");
        s1p = new JButton("1");//((Icon)createImageIcon("images/s1p.png",""));
        s1m = new JButton("-1");//((Icon)createImageIcon("images/s1m.png",""));
        s2p = new JButton("2");//((Icon)createImageIcon("images/s2p.png",""));
        s2m = new JButton("-2");//((Icon)createImageIcon("images/s2m.png",""));
        s3p = new JButton("3");//((Icon)createImageIcon("images/s3p.png",""));
        s3m = new JButton("-3");//((Icon)createImageIcon("images/s3m.png",""));
        s4p = new JButton("4");//((Icon)createImageIcon("images/s4p.png",""));
        s4m = new JButton("-4");//((Icon)createImageIcon("images/s4m.png",""));
        s5p = new JButton("5");//((Icon)createImageIcon("images/s5p.png",""));
        s5m = new JButton("-5");//((Icon)createImageIcon("images/s5m.png",""));
//        generatorsPanel.add(createButton(s0,700,"identity"));

        Dimension d = new Dimension(0,5);

        generatorsPanel.add(createButton(s1p,710,"s1 positive"));
        generatorsPanel.add(createButton(s1m,711,"s1 negative"));
        generatorsPanel.add(Box.createRigidArea(d));
        generatorsPanel.add(createButton(s2p,720,"s2 positive"));
        generatorsPanel.add(createButton(s2m,721,"s2 negative"));
        generatorsPanel.add(Box.createRigidArea(d));
        generatorsPanel.add(createButton(s3p,730,"s3 positive"));
        generatorsPanel.add(createButton(s3m,731,"s3 negative"));
        generatorsPanel.add(Box.createRigidArea(d));
        generatorsPanel.add(createButton(s4p,740,"s4 positive"));
        generatorsPanel.add(createButton(s4m,741,"s4 negative"));
        generatorsPanel.add(Box.createRigidArea(d));
        generatorsPanel.add(createButton(s5p,750,"s5 positive"));
        generatorsPanel.add(createButton(s5m,751,"s5 negative"));

        generatorsPanel.add(Box.createRigidArea(d));

        resetButton = new JButton((Icon)createImageIcon("images/reset_t.png",""));
        resetButton.setMnemonic(KeyEvent.VK_R);
        generatorsPanel.add(createButton(resetButton,699,"Reset braid"));

        cancelButton = new JButton((Icon)createImageIcon("images/cancel_t.png",""));
        cancelButton.setMnemonic(KeyEvent.VK_C);
        generatorsPanel.add(createButton(cancelButton,688,"Cancel braid"));

        handleButton = new JButton((Icon)createImageIcon("images/play_t.png",""));
        handleButton.setMnemonic(KeyEvent.VK_H);
        generatorsPanel.add(createButton(handleButton,840,"Play"));

        panel.add(generatorsPanel);

        panel.add(Box.createVerticalGlue());

        String path = "images/mate_small.png";
        Icon image = (Icon)createImageIcon(path,"matematita");
        JLabel mate = new JLabel(image, BoxLayout.X_AXIS);
        panel.add(Box.createVerticalGlue());
        panel.add(mate);

        return panel;
    }
    
    @Override
    public void mouseWheelMoved(MouseWheelEvent evt) {
        int newZoom = (int)zoom.getZoom() - evt.getWheelRotation();
        setParameter(11, newZoom);
    }

}
public class Braid extends ModelApplet {

    public Braid() {super(new BraidModel());}

    @Override
    public void init() {
//        ProjectionModel projModel = (ProjectionModel)model;

        BraidModel.MAX_ZOOM = getIntParameter("MAX_ZOOM", 80);
        BraidModel.MIN_ZOOM = getIntParameter("MIN_ZOOM", 10);
        BraidModel.INIT_ZOOM = getIntParameter("INIT_ZOOM", 40);
        BraidModel.POINTS_IN_INTERVAL = getIntParameter("POINTS_IN_INTERVAL", 10);
//        projModel.cameraX = getDoubleParameter("cameraX", 4);
//        projModel.cameraY = getDoubleParameter("cameraY", 3);
//        projModel.cameraZ = getDoubleParameter("cameraZ", 8);
//        projModel.lookAtX = getDoubleParameter("lookAtX", 0); 
//        projModel.lookAtY = getDoubleParameter("lookAtY", 0);
//        projModel.lookAtZ = getDoubleParameter("lookAtZ", 0);  
//        
//        getFloatVector(projModel.light_ambient, 3, "light_ambient");
//        getFloatVector(projModel.light_diffuse, 3, "light_diffuse");
//        getFloatVector(projModel.light_position, 4, "light_position");
                
        super.init();
    }

        
    public static void main(String[] args) {
        Model model = new BraidModel();
        final ModelFrame app = new ModelFrame(model);
        app.setVisible(true);
    }
}